
/*
WIN32 Winsock client sample code
Copyright (C) 2000  Antti Markus (antti@pld.ttu.ee)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#ifndef __GNUC__
//useful defines
#define STRICT
#define WIN32_LEAN_AND_MEAN
#endif

//standard includes
#include <windows.h>
#include <winsock.h>
#include <stdio.h>
#include <stdlib.h>

//additional includes
#include "resource.h"
#include "comlib.h"

//network notification message
#define WM_NET  WM_USER+0x100+5

//main instance handle
HINSTANCE hMainInstance;

//main window handle
HWND hMainWnd;

//winsock startup data
WSADATA wsaData;

//socket
SOCKET sClient=INVALID_SOCKET;

//address info
SOCKADDR_IN LocalAddr, RemoteAddr;

//receive buffer - 2kB for now
const int MaxBufLen=2048;
char pBuf[MaxBufLen];

int nState=0;

//handle to edit control on the dialog
HWND hMemo=0;

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
                   LPSTR lpszArgs, int nWinMode);
BOOL CALLBACK DialogFunc(HWND hdwnd, UINT Msg, WPARAM wParam, LPARAM lParam);

int WINAPI WinMain(HINSTANCE hInstance,
                   HINSTANCE hPrevInstance,
                   LPSTR lpszArgs,
                   int nWinMode)
{
  int nRes=DialogBox(hMainInstance, MAKEINTRESOURCE(IDD_DIALOG_MAIN), 0, DialogFunc);
  return 0;
}

BOOL CALLBACK DialogFunc(HWND hdwnd, UINT Msg, WPARAM wParam, LPARAM lParam)
{
  int AddrLen=sizeof(SOCKADDR);
  int nRet=0;   //return value for various functions

  switch(Msg)
  {
    case WM_INITDIALOG:
    {
      //modify system menu
      //(remove Restore, Maximize and Resize)
      //NB! If you remove Move from system menu then 
      //the window can't be moved
      HMENU hSysMenu=GetSystemMenu(hdwnd, FALSE);
      DeleteMenu(hSysMenu, 0, MF_BYPOSITION);
      DeleteMenu(hSysMenu, 1, MF_BYPOSITION);
      DeleteMenu(hSysMenu, 2, MF_BYPOSITION);
      hMainWnd=hdwnd;   //store main window handle

      //start up winsock (we are expecting version 1.1 winsock DLL)
      nRet=WSAStartup(MAKEWORD(1, 1), &wsaData);
      if(nRet)
      {
        WSAError(hdwnd, SocketErrorDesc(WSAGetLastError()));
        DestroyWindow(hdwnd);   //exit program
      }

      //check version compliance
      if((LOBYTE(wsaData.wVersion) != 1) || (HIBYTE(wsaData.wVersion) != 1))
      {
        MessageBox(hdwnd, "Winsock version not suitable", "Winsock Error", 
                   MB_OK | MB_ICONEXCLAMATION);
        DestroyWindow(hdwnd);   //exit program
      }

      //create socket
      sClient=socket(AF_INET, SOCK_STREAM, 0);

      //if socket creation failed, exit
      if(sClient == INVALID_SOCKET)
      {
        WSAError(hdwnd, SocketErrorDesc(WSAGetLastError()));
        DestroyWindow(hdwnd);
      }

      //sensitizing socket to events
      //socket will react to receive, send, connect and close events
      long lMsgs=FD_READ | FD_WRITE | FD_CONNECT | FD_CLOSE;
      nRet=WSAAsyncSelect(sClient, hMainWnd, WM_NET, lMsgs);
      if(nRet == SOCKET_ERROR)
        WSAError(hdwnd, SocketErrorDesc(WSAGetLastError()));

      //set dialog icon
      SetClassLong(hdwnd, GCL_HICON, (LONG)LoadIcon(NULL, IDI_APPLICATION));

      //get edit box handle
      hMemo=GetDlgItem(hdwnd, IDC_MEMO);
      
      //empty edit box
      SetDlgItemText(hdwnd, IDC_MEMO, "");
      
      return TRUE;
    }

    case WM_NET:
    {
      unsigned WSAEvent=WSAGETSELECTEVENT(lParam);  //extract event from lParam
      switch(WSAEvent)
      {
        //connection established!
        case FD_CONNECT:
          WriteStr(hMemo, "Connected...\r\n");
          EnableWindow(GetDlgItem(hdwnd, IDC_BUTTON_CONNECT), FALSE);
          nState=1;
          break;

        //other side is sending
        case FD_READ:
          nRet=recv(sClient, pBuf, MaxBufLen, 0);
          if(nRet == SOCKET_ERROR)
            WSAError(hdwnd, SocketErrorDesc(WSAGetLastError()));
          else
          {
            WriteStr(hMemo, "-->");
            WriteStr(hMemo, pBuf);
          }

          //fall through to send, because FD_WRITE notification only appears
          //after unfinished send operation, NOT after a successful receive operation

        //we are sending
        case FD_WRITE:
          if((nState == 2) && (pBuf[0] == '2'))
          {
            nRet=send(sClient, "QUERY name\r\n", strlen("QUERY name\r\n")+1, 0);
            break;
          }
          if(nState == 1)
          {
            WriteStr(hMemo, "Ready to send..\r\n");
            strcpy(pBuf, "GET Stuff...\r\n");
            nRet=send(sClient, pBuf, strlen(pBuf)+1, 0);
            if(nRet == SOCKET_ERROR)
              WSAError(hdwnd, SocketErrorDesc(WSAGetLastError()));
            WriteStr(hMemo, "Request sent\r\n");
            nState=2;
            WriteStr(hMemo, "State is 2\r\n");
            break;
          }
          break;

        //connection closed
        case FD_CLOSE:
          shutdown(sClient, 1);
          closesocket(sClient);
          sClient=INVALID_SOCKET;
          EnableWindow(GetDlgItem(hdwnd, IDC_BUTTON_CONNECT), TRUE);
          break;

        default:
          break;
      }
      return TRUE;
    }

    case WM_COMMAND:
      switch(LOWORD(wParam))
      {
        //connect button pressed
        case IDC_BUTTON_CONNECT:
          RemoteAddr.sin_family=PF_INET;
          //connecting to localhost
          RemoteAddr.sin_addr.s_addr=inet_addr("127.0.0.1");
          //port 5555
          RemoteAddr.sin_port=htons(5555);          
          nRet=connect(sClient, (PSOCKADDR)&RemoteAddr, AddrLen);
          if(nRet == SOCKET_ERROR)
          {
            int WSAErr=WSAGetLastError();
            //WSAEWOULDBLOCK error is ignored because our socket is non-blocking
            //anyway
            if(WSAErr != WSAEWOULDBLOCK)
              WSAError(hdwnd, SocketErrorDesc(WSAGetLastError()));
          }
          return TRUE;

        case IDOK:
        case IDCANCEL:
          DestroyWindow(hdwnd);
          return TRUE;
      }
      break;

    case WM_DESTROY:
      //clean up after ourselves

      //close sockets
      if(sClient != INVALID_SOCKET)
      {
        shutdown(sClient, 1);
        closesocket(sClient);
      }  

      //shut down winsock
      WSACleanup();
      EndDialog(hdwnd, 0);
      return TRUE;
  }
  return FALSE;
}

