
/*
Simple auxiliary functions for WIN32 applications
Copyright (C) 2000 Antti Markus (antti@pld.ttu.ee)

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <winsock.h>
#include "comlib.h"

void WriteStr(HWND hwnd, char* Str)
{
  //get text length
  int nLen=SendMessage(hwnd, WM_GETTEXTLENGTH, 0, 0);
  //select text
  SendMessage(hwnd, EM_SETSEL, (WPARAM)nLen, (LPARAM)nLen);
  //replace selection
  SendMessage(hwnd, EM_REPLACESEL, 0, (LPARAM)(LPSTR)Str);
}

int IsStr(const char* Str1, const char* Str2)
{
  int nLen=strlen(Str2);
  for(int i=0; i < nLen; i++)
  {
    if(*(Str1+i) != *(Str2+i))
      return 0;
  }
  return 1;
}

void WSAError(HWND hwnd, const char* pError)
{
  MessageBox(hwnd, pError, "Winsock Error", MB_ICONEXCLAMATION | MB_OK);
}

char* SocketErrorDesc(const int Error)
{
  switch(Error)
  {
    case WSAEINTR:
      return "Interrupted system call";
    case WSAEBADF:
      return "Bad file number";
    case WSAEACCES:
      return "Permission denied";
    case WSAEFAULT:
      return "Bad address";
    case WSAEINVAL:
      return "Invalid argument";
    case WSAEMFILE:
      return "Too many open files";
    case WSAEWOULDBLOCK:
      return "Operation would block";
    case WSAEINPROGRESS:
      return "Operation now in progress";
    case WSAEALREADY:
      return "Operation already in progress";
    case WSAENOTSOCK:
      return "Socket operation on non-socket";
    case WSAEDESTADDRREQ:
      return "Destination address required";
    case WSAEMSGSIZE:
      return "Message too long";
    case WSAEPROTOTYPE:
      return "Protocol wrong type for socket";
    case WSAENOPROTOOPT:
      return "Protocol not available";
    case WSAEPROTONOSUPPORT:
      return "Protocol not supported";
    case WSAESOCKTNOSUPPORT:
      return "Socket type not supported";
    case WSAEOPNOTSUPP:
      return "Operation not supported on socket";
    case WSAEPFNOSUPPORT:
      return "Protocol family not supported";
    case WSAEAFNOSUPPORT:
      return "Address family not supported by protocol family";
    case WSAEADDRINUSE:
      return "Address already in use";
    case WSAEADDRNOTAVAIL:
      return "Can""t assign requested address";
    case WSAENETDOWN:
      return "Network is down";
    case WSAENETUNREACH:
      return "Network is unreachable";
    case WSAENETRESET:
      return "Network dropped connection on reset";
    case WSAECONNABORTED:
      return "Software caused connection abort";
    case WSAECONNRESET:
      return "Connection reset by peer";
    case WSAENOBUFS:
      return "No buffer space available";
    case WSAEISCONN:
      return "Socket is already connected";
    case WSAENOTCONN:
      return "Socket is not connected";
    case WSAESHUTDOWN:
      return "Can""t send after socket shutdown";
    case WSAETOOMANYREFS:
      return "Too many references: can""t splice";
    case WSAETIMEDOUT:
      return "Connection timed out";
    case WSAECONNREFUSED:
      return "Connection refused";
    case WSAELOOP:
      return "Too many levels of symbolic links";
    case WSAENAMETOOLONG:
      return "File name too long";
    case WSAEHOSTDOWN:
      return "Host is down";
    case WSAEHOSTUNREACH:
      return "No route to host";
    case WSAENOTEMPTY:
      return "Directory not empty";
    case WSAEPROCLIM:
      return "Too many processes";
    case WSAEUSERS:
      return "Too many users";
    case WSAEDQUOT:
      return "Disc quota exceeded";
    case WSAESTALE:
      return "Stale NFS file handle";
    case WSAEREMOTE:
      return "Too many levels of remote in path";
    case WSASYSNOTREADY:
      return "Network sub-system is unusable";
    case WSAVERNOTSUPPORTED:
      return "WinSock DLL cannot support this application";
    case WSANOTINITIALISED:
      return "WinSock not initialized";
    case WSAHOST_NOT_FOUND:
      return "Host not found";
    case WSATRY_AGAIN:
      return "Non-authoritative host not found";
    case WSANO_RECOVERY:
      return "Non-recoverable error";
    case WSANO_DATA:
      return "No Data";
    default:
      return "Not a WinSock error";
  }
  return 0;
}
