
/*
WIN32 Winsock server sample code
Copyright (C) 2000  Antti Markus (antti@pld.ttu.ee)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#ifndef __GNUC__
//useful defines
#define STRICT
#define WIN32_LEAN_AND_MEAN
#endif

//standard includes
#include <windows.h>
#include <winsock.h>
#include <stdio.h>
#include <stdlib.h>

//additional includes
#include "resource_s.h"
#include "comlib.h"

//network notification message
#define WM_NET  WM_USER+0x100+5

//winsock data
WSADATA wsaData;

//sockets
SOCKET sListen=INVALID_SOCKET, sClient=INVALID_SOCKET;

//address info
SOCKADDR_IN LocalAddr, RemoteAddr;

//send and receive buffer - 2kB for now
const int MaxBufLen=2048;
char pBuf[MaxBufLen];
int nState=0;

HINSTANCE hMainInstance;
HWND hMainWnd, hMemo;

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
                   LPSTR lpszArgs, int nWinMode);
BOOL CALLBACK DialogFunc(HWND hdwnd, UINT Msg, WPARAM wParam, LPARAM lParam);

int WINAPI WinMain(HINSTANCE hInstance,
                   HINSTANCE hPrevInstance,
                   LPSTR lpszArgs,
                   int nWinMode)
{
  int nRes=DialogBox(hMainInstance, MAKEINTRESOURCE(IDD_DIALOG_MAIN), 0, DialogFunc);
  return 0;
}

BOOL CALLBACK DialogFunc(HWND hdwnd, UINT Msg, WPARAM wParam, LPARAM lParam)
{
  int AddrLen=sizeof(SOCKADDR);
  int nRet=0;
  switch(Msg)
  {
    case WM_INITDIALOG:
    {
      //modify system menu
      //(remove Restore, Maximize and Resize)
      //NB! If you remove Move from system menu then 
      //the window can't be moved
      HMENU hSysMenu=GetSystemMenu(hdwnd, FALSE);
      DeleteMenu(hSysMenu, 0, MF_BYPOSITION);
      DeleteMenu(hSysMenu, 1, MF_BYPOSITION);
      DeleteMenu(hSysMenu, 2, MF_BYPOSITION);

      hMainWnd=hdwnd;   //store main window handle
      hMemo=GetDlgItem(hdwnd, IDC_MEMO);  //get edit box handle
      
      //empty edit box
      SetDlgItemText(hdwnd, IDC_MEMO, "");

      //start up winsock (we are expecting version 1.1)
      int nRes=WSAStartup(MAKEWORD(1, 1), &wsaData);

      //if startup failed - exit
      if(nRes)
      {
        WSAError(hdwnd, SocketErrorDesc(WSAGetLastError()));
        DestroyWindow(hdwnd); //exit program
      }

      //create socket
      sListen=socket(AF_INET, SOCK_STREAM, 0);
      if(sListen == INVALID_SOCKET)
      {
        WSAError(0, SocketErrorDesc(WSAGetLastError()));
        DestroyWindow(hdwnd); //exit program
      }

      LocalAddr.sin_family=PF_INET;
      //server listens on port 5555
      LocalAddr.sin_port=htons(5555);

      //bind socket to address
      nRet=bind(sListen, (PSOCKADDR)&LocalAddr, AddrLen);
      if(nRet == SOCKET_ERROR)
      {
        WSAError(0, SocketErrorDesc(WSAGetLastError()));
        DestroyWindow(hdwnd);
      }
      
      //server starts up - max 5 clients at the same time
      nRet=listen(sListen, 5);
      if(nRet == SOCKET_ERROR)
      {
        WSAError(0, SocketErrorDesc(WSAGetLastError()));
        DestroyWindow(hdwnd);
      }

      //sensitize socket to network events
      //(accept, close, receive and send events)
      nRet=WSAAsyncSelect(sListen, hMainWnd, WM_NET,
                     FD_ACCEPT | FD_CLOSE | FD_READ | FD_WRITE);

      if(nRet == SOCKET_ERROR)
        WSAError(0, SocketErrorDesc(WSAGetLastError()));

      //set dialog icon
      SetClassLong(hdwnd, GCL_HICON, (LONG)LoadIcon(NULL, IDI_APPLICATION));

      WriteStr(hMemo, "Setup complete\r\n");
      return TRUE;
    }
    case WM_NET:
    {
      unsigned WSAEvent=WSAGETSELECTEVENT(lParam);  //extract event from lParam
      switch(WSAEvent)
      {
        //connection accepted!
        case FD_ACCEPT:
          //create client socket
          sClient=accept(sListen, (PSOCKADDR)&RemoteAddr, &AddrLen);
          WriteStr(hMemo, "Connection accepted\r\n");
          return TRUE;

        //client is sending
        case FD_READ:
          nRet=recv(sClient, pBuf, MaxBufLen, 0);
          WriteStr(hMemo, "Got an answer\r\n");
          if(IsStr(pBuf, "GET"))
          {
            nState=1;
            WriteStr(hMemo, "State is 1\r\n");
          }
          if(IsStr(pBuf, "QUERY"))
          {
            nState=2;
            WriteStr(hMemo, "State is 2\r\n");
          }

        //server is sending
        case FD_WRITE:
          if(!nState)
          {
            strcpy(pBuf, "250 OK\r\n");
            WriteStr(hMemo, "Send OK\r\n");
            nRet=send(sClient, pBuf, strlen(pBuf)+1, 0);
            break;
          }
          if(nState == 1)
          {
            WriteStr(hMemo, "Sending back...\r\n");
            nRet=send(sClient, pBuf, strlen(pBuf)+1, 0);
            break;
          }
          if(nState == 2)
          {
            nRet=send(sClient, "What's it to you?!\r\n", strlen("What's it to you?!\r\n")+1, 0);
            WriteStr(hMemo, "Sending QUERY answer\r\n");
            break;
          }
          break;

        case FD_CLOSE:
          //shut down client socket
          shutdown(sClient, 1);
          closesocket(sClient);
          sClient=INVALID_SOCKET;
          WriteStr(hMemo, "Connection closed\r\n");
          break;
      }
      return TRUE;
    }

    case WM_COMMAND:
      switch(LOWORD(wParam))
      {
        case IDOK:
        case IDCANCEL:
          DestroyWindow(hdwnd);
          return TRUE;
      }
      break;

    case WM_DESTROY:
      //clean up after ourselves

      //if we have a connected client
      //kick it
      if(sClient != INVALID_SOCKET)
      {
        shutdown(sClient, 1);
        closesocket(sClient);
        sClient=INVALID_SOCKET;
      }

      if(sListen != INVALID_SOCKET)
      {
        //shut down listening socket
        shutdown(sListen, 1);
        closesocket(sListen);
      }

      WSACleanup();
      EndDialog(hdwnd, 0);
      return TRUE;
  }
  return FALSE;
}

